from enum import Enum


class StrategyAttributeDeviceName(Enum):
    Browser = 'browser'
    Mobile = 'mobile'
    Desktop = 'desktop'
    Server = 'server'
    Watch = 'watch'
    Embedded = 'embedded'
