from enum import Enum


class StrategyAttributeCountryName(Enum):
    Afghanistan = 'afghanistan'
    Albania = 'albania'
    Algeria = 'algeria'
    Andorra = 'andorra'
    Angola = 'angola'
    AntiguaAndBarbuda = 'antigua_and_barbuda'
    Argentina = 'argentina'
    Armenia = 'armenia'
    Australia = 'australia'
    Austria = 'austria'
    Azerbaijan = 'azerbaijan'
    TheBahamas = 'the_bahamas'
    Bahrain = 'bahrain'
    Bangladesh = 'bangladesh'
    Barbados = 'barbados'
    Belarus = 'belarus'
    Belgium = 'belgium'
    Belize = 'belize'
    Benin = 'benin'
    Bhutan = 'bhutan'
    Bolivia = 'bolivia'
    BosniaAndHerzegovina = 'bosnia_and_herzegovina'
    Botswana = 'botswana'
    Brazil = 'brazil'
    Brunei = 'brunei'
    Bulgaria = 'bulgaria'
    BurkinaFaso = 'burkina_faso'
    Burundi = 'burundi'
    CaboVerde = 'cabo_verde'
    Cambodia = 'cambodia'
    Cameroon = 'cameroon'
    Canada = 'canada'
    CentralAfricanRepublic = 'central_african_republic'
    Chad = 'chad'
    Chile = 'chile'
    China = 'china'
    Colombia = 'colombia'
    Comoros = 'comoros'
    CongoDemocraticRepublicOfThe = 'congo_democratic_republic_of_the'
    CongoRepublicOfThe = 'congo_republic_of_the'
    CostaRica = 'costa_rica'
    CoteDivoire = 'cote_divoire'
    Croatia = 'croatia'
    Cuba = 'cuba'
    Cyprus = 'cyprus'
    CzechRepublic = 'czech_republic'
    Denmark = 'denmark'
    Djibouti = 'djibouti'
    Dominica = 'dominica'
    DominicanRepublic = 'dominican_republic'
    EastTimor = 'east_timor'
    Ecuador = 'ecuador'
    Egypt = 'egypt'
    ElSalvador = 'el_salvador'
    EquatorialGuinea = 'equatorial_guinea'
    Eritrea = 'eritrea'
    Estonia = 'estonia'
    Eswatini = 'eswatini'
    Ethiopia = 'ethiopia'
    Fiji = 'fiji'
    Finland = 'finland'
    France = 'france'
    Gabon = 'gabon'
    TheGambia = 'the_gambia'
    Georgia = 'georgia'
    Germany = 'germany'
    Ghana = 'ghana'
    Greece = 'greece'
    Grenada = 'grenada'
    Guatemala = 'guatemala'
    Guinea = 'guinea'
    GuineaBissau = 'guinea_bissau'
    Guyana = 'guyana'
    Haiti = 'haiti'
    Honduras = 'honduras'
    Hungary = 'hungary'
    Iceland = 'iceland'
    India = 'india'
    Indonesia = 'indonesia'
    Iran = 'iran'
    Iraq = 'iraq'
    Ireland = 'ireland'
    Israel = 'israel'
    Italy = 'italy'
    Jamaica = 'jamaica'
    Japan = 'japan'
    Jordan = 'jordan'
    Kazakhstan = 'kazakhstan'
    Kenya = 'kenya'
    Kiribati = 'kiribati'
    KoreaNorth = 'korea_north'
    KoreaSouth = 'korea_south'
    Kosovo = 'kosovo'
    Kuwait = 'kuwait'
    Kyrgyzstan = 'kyrgyzstan'
    Laos = 'laos'
    Latvia = 'latvia'
    Lebanon = 'lebanon'
    Lesotho = 'lesotho'
    Liberia = 'liberia'
    Libya = 'libya'
    Liechtenstein = 'liechtenstein'
    Lithuania = 'lithuania'
    Luxembourg = 'luxembourg'
    Madagascar = 'madagascar'
    Malawi = 'malawi'
    Malaysia = 'malaysia'
    Maldives = 'maldives'
    Mali = 'mali'
    Malta = 'malta'
    MarshallIslands = 'marshall_islands'
    Mauritania = 'mauritania'
    Mauritius = 'mauritius'
    Mexico = 'mexico'
    MicronesiaFederatedStatesOf = 'micronesia_federated_states_of'
    Moldova = 'moldova'
    Monaco = 'monaco'
    Mongolia = 'mongolia'
    Montenegro = 'montenegro'
    Morocco = 'morocco'
    Mozambique = 'mozambique'
    Myanmar = 'myanmar'
    Namibia = 'namibia'
    Nauru = 'nauru'
    Nepal = 'nepal'
    Netherlands = 'netherlands'
    NewZealand = 'new_zealand'
    Nicaragua = 'nicaragua'
    Niger = 'niger'
    Nigeria = 'nigeria'
    NorthMacedonia = 'north_macedonia'
    Norway = 'norway'
    Oman = 'oman'
    Pakistan = 'pakistan'
    Palau = 'palau'
    Panama = 'panama'
    PapuaNewGuinea = 'papua_new_guinea'
    Paraguay = 'paraguay'
    Peru = 'peru'
    Philippines = 'philippines'
    Poland = 'poland'
    Portugal = 'portugal'
    Qatar = 'qatar'
    Romania = 'romania'
    Russia = 'russia'
    Rwanda = 'rwanda'
    SaintKittsAndNevis = 'saint_kitts_and_nevis'
    SaintLucia = 'saint_lucia'
    SaintVincentAndTheGrenadines = 'saint_vincent_and_the_grenadines'
    Samoa = 'samoa'
    SanMarino = 'san_marino'
    SaoTomeAndPrincipe = 'sao_tome_and_principe'
    SaudiArabia = 'saudi_arabia'
    Senegal = 'senegal'
    Serbia = 'serbia'
    Seychelles = 'seychelles'
    SierraLeone = 'sierra_leone'
    Singapore = 'singapore'
    Slovakia = 'slovakia'
    Slovenia = 'slovenia'
    SolomonIslands = 'solomon_islands'
    Somalia = 'somalia'
    SouthAfrica = 'south_africa'
    Spain = 'spain'
    SriLanka = 'sri_lanka'
    Sudan = 'sudan'
    SudanSouth = 'sudan_south'
    Suriname = 'suriname'
    Sweden = 'sweden'
    Switzerland = 'switzerland'
    Syria = 'syria'
    Taiwan = 'taiwan'
    Tajikistan = 'tajikistan'
    Tanzania = 'tanzania'
    Thailand = 'thailand'
    Togo = 'togo'
    Tonga = 'tonga'
    TrinidadAndTobago = 'trinidad_and_tobago'
    Tunisia = 'tunisia'
    Turkey = 'turkey'
    Turkmenistan = 'turkmenistan'
    Tuvalu = 'tuvalu'
    Uganda = 'uganda'
    Ukraine = 'ukraine'
    UnitedArabEmirates = 'united_arab_emirates'
    UnitedKingdom = 'united_kingdom'
    UnitedStates = 'united_states'
    Uruguay = 'uruguay'
    Uzbekistan = 'uzbekistan'
    Vanuatu = 'vanuatu'
    VaticanCity = 'vatican_city'
    Venezuela = 'venezuela'
    Vietnam = 'vietnam'
    Yemen = 'yemen'
    Zambia = 'zambia'
    Zimbabwe = 'zimbabwe'
